#include <windows.h> 
#include "gpioLib.h"
#include "spilib.h"
#include "coproclib.h"
 
#define DATA_BITS 16
#define DURATION_FOR_WAVEFORM 40																								///< Loop for generating wave for certain time
typedef enum {
	SINE_WAVE		= 'a',
	SQUARE_WAVE		= 'b',
	SAW_WAVE		= 'c',
	TRAINGLE_WAVE	= 'd',
	EXIT_OPTION		= 'e'
}option;
char sineWave[129] = {	0x63, 0x38, 0x2B, 0x39, 0xF1, 0x39, 0xB1, 0x3A, 0x6A, 0x3B, 0x1C, 0x3C, 0xC3, 0x3C, 0x5E, 0x3D,
						0xEC, 0x3D, 0x6C, 0x3E, 0xDB, 0x3E, 0x3A, 0x3F, 0x87, 0x3F, 0xC1, 0x3F, 0xE8, 0x3F, 0xFC, 0x3F, 
						0xFC, 0x3F, 0xE8, 0x3F, 0xC1, 0x3F, 0x87, 0x3F, 0x3A, 0x3F, 0xDB, 0x3E, 0x6C, 0x3E, 0xEC, 0x3D, 
						0x5E, 0x3D, 0xC3, 0x3C, 0x1C, 0x3C, 0x6A, 0x3B, 0xB1, 0x3A, 0xF1, 0x39, 0x2B, 0x39, 0x63, 0x38, 
						0x9B, 0x37, 0xD3, 0x36, 0x0D, 0x36, 0x4D, 0x35, 0x94, 0x34, 0xE2, 0x33, 0x3B, 0x33, 0xA0, 0x32, 
						0x12, 0x32, 0x92, 0x31, 0x23, 0x31, 0xC4, 0x30, 0x77, 0x30, 0x3d, 0x30, 0x16, 0x30, 0x02, 0x30, 
						0x02, 0x30, 0x16, 0x30, 0x3D, 0x30, 0x77, 0x30, 0xC4, 0x30, 0x23, 0x31, 0x92, 0x31, 0x12, 0x32, 
						0xA0, 0x32, 0x3B, 0x33, 0xE2, 0x33, 0x94, 0x34, 0x4D, 0x35, 0x0D, 0x36, 0xD3, 0x36, 0x9B, 0x37,
						0x00};																									///< SineWave data value 

char sawWave[69] =	 {	0x01, 0x30, 0x7D, 0x30, 0xFA, 0x30, 0x77, 0x31, 0xF4, 0x31, 0x71, 0x32, 0xEE, 0x32, 0x6B, 0x33,
						0xE8, 0x33, 0x65, 0x34, 0xE2, 0x34, 0x5F, 0x35, 0xDC, 0x35, 0x59, 0x36, 0xD6, 0x36, 0x53, 0x37, 
						0xD0, 0x37, 0x4D, 0x38, 0xCA, 0x38, 0x47, 0x39, 0xC4, 0x39, 0x41, 0x3A, 0xBE, 0x3A, 0x3B, 0x3B,
						0xB8, 0x3B, 0x35, 0x3C, 0xB2, 0x3C, 0x2F, 0x3D, 0xAC, 0x3D, 0x29, 0x3E, 0xA6, 0x3E, 0x23, 0x3F,	
						0xA0, 0x3F, 0x01, 0x30, 0x00};																			///< Saw Wave data Value

char triangleWave[131]={0x01, 0x30, 0x7D, 0x30, 0xFA, 0x30, 0x77, 0x31, 0xF4, 0x31, 0x71, 0x32, 0xEE, 0x32, 0x6B, 0x33, 
						0xE8, 0x33, 0x65, 0x34, 0xE2, 0x34, 0x5F, 0x35, 0xDC, 0x35, 0x59, 0x36, 0xD6, 0x36, 0x53, 0x37, 
						0xD0, 0x37, 0x4D, 0x38, 0xCA, 0x38, 0x47, 0x39, 0xC4, 0x39, 0x41, 0x3A, 0xBE, 0x3A, 0x3B, 0x3B, 
						0xB8, 0x3B, 0x35, 0x3C, 0xB2, 0x3C, 0x2F, 0x3D, 0xAC, 0x3D, 0x29, 0x3E, 0xA6, 0x3E, 0x23, 0x3F, 
						0xA0, 0x3F, 0x23, 0x3F, 0xA6, 0x3E, 0x29, 0x3E, 0xAC, 0x3D, 0x2F, 0x3D, 0xB2, 0x3C, 0x35, 0x3C, 
						0xB8, 0x3B, 0x3B, 0x3B, 0xBE, 0x3A, 0x41, 0x3A, 0xC4, 0x39, 0x47, 0x39, 0xCA, 0x38, 0x4D, 0x38, 
						0xD0, 0x37, 0x53, 0x37, 0xD6, 0x36, 0x59, 0x36, 0xDC, 0x35, 0x5F, 0x35, 0xE2, 0x34, 0x65, 0x34, 
						0xE8, 0x33, 0x6B, 0x33, 0xEE, 0x32, 0x71, 0x32, 0xF4, 0x31, 0x77, 0x31, 0xFA, 0x30, 0x7D, 0x30,
						0x01, 0x30, 0x00};																						///< Triangle Wave data Value

char squareWave[6] =   {0xd9, 0x34, 0x01, 0x30, 0x00};																			///< Square Wave data 

///< Write Wave form digital data value to DAC chip over SPI and generate corresponding waveform
BOOL generateWaveForm(DWORD txSSPport, CHAR *waveFormValue, DWORD loopCounter)
{
	DWORD	loopCount	= 0;
	CHAR    spiData[2]	= {0};
	DWORD	dataCount   = 0;
	DWORD   size        = strlen(waveFormValue);										///< Get buffer size
	while(loopCount < loopCounter)
	{
		for (dataCount = 0; dataCount < size; dataCount = dataCount+2)
		{
			spiData[0] = waveFormValue[dataCount];
			spiData[1] = waveFormValue[dataCount+1];									///< Prepare buffer to transmit over SPI
			SPIWritePolling(txSSPport, (DWORD*)spiData, 2, 0);							///< Write 2 frame over SPI
			Sleep(1);
		}
		loopCount++;
	}


	return TRUE;
}

int wmain()
{
    DWORD		procID		= 0;														///< Get Proc Id
	CHAR		choice		= ' ';														///< Console Menu option 
	DWORD		txSSPport	= 1;														///< SPI port
    procID = getPROCID();																///< Get SOC type
    if (procID == TYPE_TEGRA2)
    {
        txSSPport = 4;
        printf("Tegra detected\n");
    }
	else if (procID == TYPE_PXA270 || procID == TYPE_PXA300 || procID == TYPE_PXA310 || procID == TYPE_PXA320)
    {
        txSSPport = 1;
        printf("PXA detected\n");
    }

    if (!SPIInit(txSSPport, DATA_BITS, SPI_CLOCK_6_5_MHZ, FALSE, SPI_MODE_0, NULL))		///< Initialize SSP (master), MODE 0
    {
        printf("SPI was not able to initialize\n");
    }
    else 
    {
        printf("Initialization complete\n");
    }
	while(choice != EXIT_OPTION)
	{
		fflush(stdin);
		printf("Select Option\na.) Sine wave\nb.) Square wave\nc.) Sawtooth\nd.) Triangle wave\ne.) Exit\n");
		scanf_s("%c", &choice);
		switch(choice)
		{
		case SINE_WAVE:
			generateWaveForm(txSSPport, sineWave, DURATION_FOR_WAVEFORM);					///< Sine Wave 
			break;
		case SQUARE_WAVE:
			generateWaveForm(txSSPport, squareWave, DURATION_FOR_WAVEFORM);					///< Square Wave
			break;
		case SAW_WAVE:
			generateWaveForm(txSSPport, sawWave, DURATION_FOR_WAVEFORM);					///< Saw Wave
			break;
		case TRAINGLE_WAVE:
			generateWaveForm(txSSPport, triangleWave, DURATION_FOR_WAVEFORM);				///< Traingle Wave
			break;
		case EXIT_OPTION:
			SPIDeinit(txSSPport);															///< De-initialize SPI master channel
			wprintf(L"Closing Demo.....");
			Sleep(2000);
			break;
		default:
			printf("Invalid option\n\n");
			break;
		}
	}
    return TRUE;
}